from __future__ import absolute_import, unicode_literals

from django.contrib.auth import get_user_model
from django.core.files import File
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase

from documents.models import DocumentType
from documents.tests import (
    TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME, TEST_ADMIN_EMAIL,
    TEST_DOCUMENT_TYPE, TEST_SMALL_DOCUMENT_PATH
)

from ..models import Folder

from .literals import TEST_FOLDER_LABEL, TEST_FOLDER_EDITED_LABEL


class FolderViewTestCase(TestCase):
    def setUp(self):
        self.admin_user = get_user_model().objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )

        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

        self.client = Client()
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.admin_user.is_authenticated())

    def tearDown(self):
        self.admin_user.delete()
        self.document_type

    def test_folder_create_view(self):
        response = self.client.post(
            reverse('folders:folder_create'), data={
                'label': TEST_FOLDER_LABEL
            }, follow=True
        )

        self.assertContains(response, text='created', status_code=200)
        self.assertEqual(Folder.objects.count(), 1)
        self.assertEqual(Folder.objects.first().label, TEST_FOLDER_LABEL)
        self.assertEqual(Folder.objects.first().user, self.admin_user)

    def test_folder_delete_view(self):
        folder = Folder.objects.create(
            label=TEST_FOLDER_LABEL, user=self.admin_user
        )

        response = self.client.post(
            reverse('folders:folder_delete', args=(folder.pk,)), follow=True
        )

        self.assertContains(response, text='deleted', status_code=200)
        self.assertEqual(Folder.objects.count(), 0)

    def test_folder_edit_view(self):
        folder = Folder.objects.create(
            label=TEST_FOLDER_LABEL, user=self.admin_user
        )

        response = self.client.post(
            reverse('folders:folder_edit', args=(folder.pk,)), data = {
                'label': TEST_FOLDER_EDITED_LABEL
            }, follow=True
        )

        folder = Folder.objects.get(pk=folder.pk)
        self.assertContains(response, text='saved', status_code=200)
        self.assertEqual(folder.label, TEST_FOLDER_EDITED_LABEL)
        self.assertEqual(folder.user, self.admin_user)

    def test_folder_add_document_view(self):
        folder = Folder.objects.create(
            label=TEST_FOLDER_LABEL, user=self.admin_user
        )

        response = self.client.post(
            reverse('folders:folder_add_document', args=(self.document.pk,)),
            data={
                'folder': folder.pk,
            }, follow=True
        )

        folder = Folder.objects.get(pk=folder.pk)
        self.assertContains(response, text='added', status_code=200)
        self.assertEqual(folder.documents.count(), 1)
        self.assertQuerysetEqual(
            folder.documents.all(), (repr(self.document),)
        )

    def test_folder_add_multiple_documents_view(self):
        folder = Folder.objects.create(
            label=TEST_FOLDER_LABEL, user=self.admin_user
        )

        response = self.client.post(
            reverse('folders:folder_add_multiple_documents'), data={
                'id_list': (self.document.pk,),
                'folder': folder.pk
            }, follow=True
        )

        folder = Folder.objects.get(pk=folder.pk)
        self.assertContains(response, text='added', status_code=200)
        self.assertEqual(folder.documents.count(), 1)
        self.assertQuerysetEqual(
            folder.documents.all(), (repr(self.document),)
        )

    def test_folder_remove_document_view(self):
        folder = Folder.objects.create(
            label=TEST_FOLDER_LABEL, user=self.admin_user,
        )

        folder.documents.add(self.document)

        self.assertEqual(folder.documents.count(), 1)

        response = self.client.post(
            reverse(
                'folders:folder_document_multiple_remove', args=(folder.pk,)
            ), data={
                'id_list': (self.document.pk,),
            }, follow=True
        )

        folder = Folder.objects.get(pk=folder.pk)
        self.assertContains(response, text='removed', status_code=200)
        self.assertEqual(folder.documents.count(), 0)



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from common import MayanAppConfig, menu_tools
from common.classes import Package

from .classes import APIEndPoint
from .links import link_api, link_api_documentation


class RESTAPIApp(MayanAppConfig):
    app_url = 'api'
    name = 'rest_api'
    verbose_name = _('REST API')

    def ready(self):
        super(RESTAPIApp, self).ready()

        APIEndPoint(app=self, name='rest', version_string='1')

        Package(label='Django REST framework', license_text='''
Copyright (c) 2011-2015, Tom Christie All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer. Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
        ''')

        Package(label='django-rest-swagger', license_text='''
Copyright (c) 2013, Marc Gibbons
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
        ''')

        menu_tools.bind_links(links=(link_api, link_api_documentation))



from __future__ import unicode_literals

TEST_GROUP = 'test group'
TEST_ROLE_LABEL = 'test role'
TEST_ROLE_LABEL_EDITED = 'test role label edited'
TEST_USER_USERNAME = 'test user'
TEST_USER_PASSWORD = 'test password'



from __future__ import absolute_import, unicode_literals

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group
from django.core.files import File
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase

from documents.models import DocumentType
from documents.permissions import permission_document_view
from documents.tests import TEST_DOCUMENT_TYPE, TEST_SMALL_DOCUMENT_PATH
from permissions import Permission
from permissions.models import Role
from permissions.tests.literals import (
    TEST_GROUP, TEST_ROLE_LABEL, TEST_USER_PASSWORD, TEST_USER_USERNAME
)

from ..models import Tag
from ..permissions import permission_tag_view

from .literals import TEST_TAG_COLOR, TEST_TAG_LABEL


class TagViewTestCase(TestCase):
    def setUp(self):
        self.user = get_user_model().objects.create_user(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )
        self.group = Group.objects.create(name=TEST_GROUP)
        self.role = Role.objects.create(label=TEST_ROLE_LABEL)
        Permission.invalidate_cache()

        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

        self.client = Client()
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_USER_USERNAME, password=TEST_USER_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.user.is_authenticated())

        self.tag = Tag.objects.create(
            color=TEST_TAG_COLOR, label=TEST_TAG_LABEL
        )

        self.tag.documents.add(self.document)

        self.group.user_set.add(self.user)
        self.role.groups.add(self.group)
        self.role.permissions.add(permission_document_view.stored_permission)

    def tearDown(self):
        self.group.delete()
        self.role.delete()
        self.tag.delete()
        self.user.delete()
        self.document_type

    def test_document_tags_widget_no_permissions(self):
        response = self.client.get(reverse('documents:document_list'))
        self.assertNotContains(response, text=TEST_TAG_LABEL, status_code=200)

    def test_document_tags_widget_with_permissions(self):
        self.role.permissions.add(permission_tag_view.stored_permission)
        response = self.client.get(reverse('documents:document_list'))
        self.assertContains(
            response, text=TEST_TAG_LABEL.replace(' ', '&nbsp;'),
            status_code=200
        )



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.core.files import File
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase

from documents.models import DocumentType
from documents.tests.literals import (
    TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME, TEST_ADMIN_EMAIL,
    TEST_DOCUMENT_TYPE, TEST_SMALL_DOCUMENT_PATH
)

from ..models import MetadataType, DocumentMetadata

from .literals import (
    TEST_DOCUMENT_TYPE_2, TEST_DOCUMENT_METADATA_VALUE_2,
    TEST_METADATA_TYPE_LABEL, TEST_METADATA_TYPE_LABEL_2,
    TEST_METADATA_TYPE_NAME, TEST_METADATA_TYPE_NAME_2
)


class DocumentMetadataTestCase(TestCase):
    def setUp(self):
        self.admin_user = User.objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )
        self.client = Client()
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.admin_user.is_authenticated())

        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        self.metadata_type = MetadataType.objects.create(
            name=TEST_METADATA_TYPE_NAME, label=TEST_METADATA_TYPE_LABEL
        )

        self.document_type.metadata.create(metadata_type=self.metadata_type)

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        self.document_type.delete()
        self.admin_user.delete()
        self.metadata_type.delete()

    def test_remove_metadata_view(self):
        document_metadata = self.document.metadata.create(
            metadata_type=self.metadata_type, value=''
        )

        self.assertEqual(len(self.document.metadata.all()), 1)

        # Test display of metadata removal form
        response = self.client.get(
            reverse(
                'metadata:metadata_remove', args=(self.document.pk,),
            )
        )

        self.assertContains(response, 'emove', status_code=200)

        # Test post to metadata removal view
        response = self.client.post(
            reverse(
                'metadata:metadata_remove', args=(self.document.pk,),
            ), data={
                'form-0-id': document_metadata.metadata_type.pk,
                'form-0-update': True,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': '',
            }, follow=True
        )

        self.assertContains(response, 'Success', status_code=200)

        self.assertEqual(len(self.document.metadata.all()), 0)

    def test_metadata_edit_after_document_type_change(self):
        # Gitlab issue #204
        # Problems to add required metadata after changin the document type

        document_type_2 = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE_2
        )

        metadata_type_2 = MetadataType.objects.create(
            name=TEST_METADATA_TYPE_NAME_2, label=TEST_METADATA_TYPE_LABEL_2
        )

        document_metadata_2 = document_type_2.metadata.create(
            metadata_type=metadata_type_2, required=True
        )

        self.document.set_document_type(document_type=document_type_2)

        response = self.client.get(
            reverse(
                'metadata:metadata_edit', args=(self.document.pk,),
            ), follow=True
        )

        self.assertContains(response, 'Edit', status_code=200)

        response = self.client.post(
            reverse(
                'metadata:metadata_edit', args=(self.document.pk,),
            ), data={
                'form-0-id': document_metadata_2.pk,
                'form-0-update': True,
                'form-0-value': TEST_DOCUMENT_METADATA_VALUE_2,
                'form-TOTAL_FORMS': '1',
                'form-INITIAL_FORMS': '0',
                'form-MAX_NUM_FORMS': '',
            }, follow=True
        )

        self.assertContains(response, 'Metadata for document', status_code=200)

        self.assertEqual(
            self.document.metadata.get(metadata_type=metadata_type_2
        ).value, TEST_DOCUMENT_METADATA_VALUE_2)



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.core.files import File
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase

from documents.tests.literals import (
    TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME, TEST_ADMIN_EMAIL
)


class CommonViewTestCase(TestCase):
    def setUp(self):
        self.admin_user = User.objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )
        self.client = Client()
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.admin_user.is_authenticated())

    def tearDown(self):
        self.admin_user.delete()

    def test_about_view(self):
        response = self.client.get(reverse('common:about_view'))
        self.assertContains(response, text='About', status_code=200)



from __future__ import unicode_literals

import datetime
import time

from django.contrib.auth.models import User
from django.core.files import File
from django.core.urlresolvers import reverse
from django.test.client import Client
from django.test import TestCase
from django.utils.timezone import now

from common.literals import TIME_DELTA_UNIT_DAYS
from documents.models import DocumentType
from documents.tests.literals import (
    TEST_ADMIN_PASSWORD, TEST_ADMIN_USERNAME, TEST_ADMIN_EMAIL,
    TEST_DOCUMENT_TYPE, TEST_SMALL_DOCUMENT_PATH
)

from ..models import DocumentCheckout


class DocumentCheckoutViewTestCase(TestCase):
    def setUp(self):
        self.admin_user = User.objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )
        self.client = Client()
        # Login the admin user
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        self.assertTrue(self.admin_user.is_authenticated())

        self.document_type = DocumentType.objects.create(
            label=TEST_DOCUMENT_TYPE
        )

        with open(TEST_SMALL_DOCUMENT_PATH) as file_object:
            self.document = self.document_type.new_document(
                file_object=File(file_object)
            )

    def tearDown(self):
        self.document_type.delete()
        self.admin_user.delete()

    def test_checkout_view(self):
        response = self.client.post(
            reverse(
                'checkouts:checkout_document', args=(self.document.pk,),
            ), data={
                'expiration_datetime_0': 2,
                'expiration_datetime_1': TIME_DELTA_UNIT_DAYS,
                'block_new_version': True
            }, follow=True
        )

        self.assertEquals(response.status_code, 200)

        self.assertTrue(self.document.is_checked_out())

        self.assertTrue(
            DocumentCheckout.objects.is_document_checked_out(
                document=self.document
            )
        )

    def test_checkin_view(self):
        expiration_datetime = now() + datetime.timedelta(days=1)

        DocumentCheckout.objects.checkout_document(
            document=self.document, expiration_datetime=expiration_datetime,
            user=self.admin_user, block_new_version=True
        )

        self.assertTrue(self.document.is_checked_out())

        response = self.client.post(
            reverse(
                'checkouts:checkin_document', args=(self.document.pk,),
            ), follow=True
        )

        self.assertEquals(response.status_code, 200)

        self.assertFalse(self.document.is_checked_out())

        self.assertFalse(
            DocumentCheckout.objects.is_document_checked_out(
                document=self.document
            )
        )
